@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 class Profile
@@ -22,27 +21,21 @@ public:
   enum Type { left, top, right, bottom, height, width };
 
 private:
-  const Block * _block;		// Block to witch this profile belongs
-  const Blockmap * _blockmap;	// Blockmap to which the block belongs
+  const Bitmap * _bitmap;	// Bitmap to witch this profile belongs
   Type type;
-  bool valid, valid_max, valid_min, valid_mean;
-  bool valid_isconcave, valid_isconvex;
-  bool valid_isflat, valid_isflats, valid_ispit, valid_istpit, valid_isupit;
-  bool valid_isvpit, valid_istip;
   int _limit, _max, _min, _mean;
-  bool _isconcave, _isconvex, _isflat, _isflats, _ispit, _istpit, _isupit;
-  bool _isvpit, _istip;
+  signed char _isconcave, _isconvex, _isflat, _isflats,
+              _ispit, _istpit, _isupit, _isvpit, _istip;
   std::vector< int > data;
   void initialize() throw();
   int mean() throw();
 
 public:
-  Profile( const Block & b, Type t ) throw();
+  Profile( const Bitmap & b, Type t ) throw();
 
-  const Block * block() const throw() { return _block; }
-  const Blockmap * blockmap() const throw() { return _blockmap; }
+//  const Bitmap * bitmap() const throw() { return _bitmap; }
 
-  int limit() throw() { if( !valid ) initialize(); return _limit; }
+  int limit() throw() { if( _limit < 0 ) initialize(); return _limit; }
   int max() throw();
   int max( int l, int r = -1 ) throw();
   int min() throw();
@@ -50,9 +43,9 @@ public:
   int operator[]( int i ) throw();
   int pos( int p ) throw() { return ( ( samples() - 1 ) * p ) / 100; }
   int range() throw() { return max() - min(); }
-  int samples() throw() { if( !valid ) initialize(); return data.size(); }
+  int samples() throw() { if( _limit < 0 ) initialize(); return data.size(); }
 
-  int area( int l = 0, int r = -1 ) throw();
+  int  area( int l = 0, int r = -1 ) throw();
   bool increasing( int i = 1 ) throw();
   bool decreasing( int i = 1 ) throw();
   bool isconcave() throw();
@@ -60,13 +53,13 @@ public:
   bool isflat() throw();
   bool isflats() throw();
   bool ispit() throw();
-  bool iscpit( int cpos = 50 ) throw();
+  bool iscpit( const int cpos = 50 ) throw();
   bool islpit() throw();
   bool istpit() throw();
   bool isupit() throw();
   bool isvpit() throw();
   bool istip() throw();
-  bool isctip( int cpos = 50 ) throw();
+  bool isctip( const int cpos = 50 ) throw();
   int  imaximum() throw();
   int  iminimum( int m = 0, int th = -1 ) throw();
   int  minima( int th = -1 ) throw();
