@@ -1,9 +1,9 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007 Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,25 +12,21 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-class Blockmap;
 class Textblock;
 
 class Textpage : public Rectangle
   {
   const std::string name;
-  const Blockmap * bmp;
   std::vector< Textblock * > tbpv;
 
 public:
-  Textpage( const Bitmap & page_image, const char * filename,
-            const Control & control ) throw();
+  Textpage( const Page_image & page_image,
+            const char * filename, const Control & control ) throw();
   ~Textpage() throw();
 
-//  const Blockmap & blockmap() const throw() { return *bmp; }
 //  const Textblock & textblock( int i ) const throw();
   int textblocks() const throw() { return tbpv.size(); }
 
